package org.matsim.core.scenario;

import org.matsim.api.core.v01.Scenario;
import org.matsim.core.config.Config;


/**
 * Provides ways to get a Scenario from the implementation in this package.
 *
 * @author michaz
 *
 */
public class ScenarioUtils {

	private ScenarioUtils() {
		// make it private, so it cannot be instantiated
	}

	/**
	 *
	 * Creates an unpopulated scenario. The configuration passed into this method is
	 * a) used to determine which containers are required, depending on the options set in the scenario config group, and
	 * b) wrapped in the Scenario
	 *
	 * User code surrenders the config to the scenario. The config should not be externally changed afterwards.
	 *
	 * @param config A {@link Config} object, must not be <code>null</code>
	 *
	 * @see org.matsim.core.config.ConfigUtils#createConfig()
	 */
	public static Scenario createScenario(final Config config) {
		if (config == null) {
			throw new NullPointerException("config must not be null!");
		}
		return new ScenarioImpl(config);
	}

	/**
	 *
	 * Initializes a scenario and populates it with data read from the input files which are named in the config.
	 *
	 */
	public static Scenario loadScenario(final Config config) {
		return ScenarioLoaderImpl.loadScenario(config);
	}

	/**
	 *
	 * Populates a scenario with data read from the input files which are named in the config which is wrapped
	 * in the scenario.
	 *
	 */
	public static void loadScenario(final Scenario scenario) {
		ScenarioLoaderImpl.loadScenario(scenario);
	}

}
